#!/usr/bin/env python
# on your machine, just run pytest in this directory or execute it to get outputs
#

import pytest

from a2p2.jmmc.models import _model


def test_same_blocks():
    """  duplicated blocks must have distinct computed names """
    m='[{ "type":"elong_gaussian", "flux_weight":1, "x":0, "y":0, "minor_axis_fwhm":1.234, "elong_ratio":1.2, "major_axis_pos_angle":45.6},{"type":"elong_gaussian", "flux_weight":1, "x":0, "y":0, "minor_axis_fwhm":1.234, "elong_ratio":1.2, "major_axis_pos_angle":45.6}]'
    xml_m=_model(m)
    assert "elong_gaussian2" in xml_m




#!/usr/bin/env python

__all__ = []

import re

from a2p2.vlti.instrument import OBConstraints
from a2p2.vlti.instrument import OBTarget
from a2p2.vlti.instrument import TSF
from a2p2.vlti.instrument import VltiInstrument

HELPTEXT = """
Please define PIONIER instrument help in a2p2/vlti/pionier.py
"""


class Pionier(VltiInstrument):

    def __init__(self, facility):
        VltiInstrument.__init__(self, facility, "PIONIER")

    def checkOB(self, ob, p2container=None):
        ui = self.ui

        instrumentConfiguration = ob.instrumentConfiguration
        BASELINE = ob.interferometerConfiguration.stations
        # Compute tel = UT or AT
        # TODO move code into common part
        if "U" in BASELINE:
            tel = "UT"
        else:
            tel = "AT"

        instrumentMode = instrumentConfiguration.instrumentMode

        # Retrieve SPEC and POL info from instrumentMode
        for disp in self.getRange("PIONIER_acq.tsf", "INS.DISP.NAME"):
            if disp in instrumentMode[0:len(disp)]:
                ins_disp = disp

        for observationConfiguration in self.getSequence(ob):

            # create keywords storage objects
            acqTSF = TSF(self, "PIONIER_acq.tsf")
            obsTSF = TSF(self, "PIONIER_obs_calibrator.tsf")
            # alias for PIONIER_obs_calibrator.tsf and
            # PIONIER_obs_science.tsf")
            kappaTSF = TSF(self, "PIONIER_gen_cal_kappa.tsf")
            darkTSF = TSF(self, "PIONIER_gen_cal_dark.tsf")

            obTarget = OBTarget()
            obConstraints = OBConstraints(self)

            # set common properties
            acqTSF.INS_DISP_NAME = ins_disp

            if 'SCIENCE' in observationConfiguration.type:
                OBJTYPE = 'SCIENCE'
            else:
                OBJTYPE = 'CALIBRATOR'

            scienceTarget = observationConfiguration.SCTarget

            # define target
            # acqTSF.SEQ_INS_SOBJ_NAME = scienceTarget.name.strip()


            obTarget.name = scienceTarget.name.strip().replace(
                ' ', '_')  # allowed characters: letters, digits, + - _ . and no spaces
            obTarget.ra, obTarget.dec = self.getCoords(scienceTarget)
            obTarget.properMotionRa, obTarget.properMotionDec = self.getPMCoords(
                scienceTarget)

            # Set baseline  interferometric array code (should be a keywordlist)
            acqTSF.ISS_BASELINE = [self.getBaselineCode(BASELINE)]

            self.checkIssVltiType(acqTSF)

            # define some default values
            DIAMETER = float(self.get(scienceTarget, "DIAMETER", 0.0))
            VIS = 1.0  # FIXME

            # Retrieve Fluxes
            TEL_COU_MAG = self.getFlux(scienceTarget, "V")
            acqTSF.ISS_IAS_HMAG = self.getFlux(scienceTarget, "H")

            # setup some default values, to be changed below
            TEL_COU_GSSOURCE = 'SCIENCE'  # by default
            GSRA = '00:00:00.000'
            GSDEC = '00:00:00.000'

            # initialize FT variables (must exist)

            # AO target
            aoTarget = ob.get(observationConfiguration, "AOTarget")
            if aoTarget != None:
                AONAME = aoTarget.name
                TEL_COU_GSSOURCE = 'SETUPFILE'  # since we have an AO
                # TODO check if AO coords should be required by template
                # AORA, AODEC  = self.getCoords(aoTarget,
                # requirePrecision=False)
                acqTSF.TEL_COU_PMA, acqTSF.TEL_COU_PMD = self.getPMCoords(
                    aoTarget)

            # Guide Star
            gsTarget = ob.get(observationConfiguration, 'GSTarget')
            if gsTarget != None:
                TEL_COU_GSSOURCE = 'SETUPFILE'  # since we have an GS
                GSRA, GSDEC = self.getCoords(gsTarget, requirePrecision=False)
                # no PMRA, PMDE for GS !!
                TEL_COU_MAG = float(gsTarget.FLUX_V)

            # LST interval
            try:
                obsConstraint = observationConfiguration.observationConstraints
                LSTINTERVAL = obsConstraint.LSTinterval
            except:
                LSTINTERVAL = None

            # Constraints
            obConstraints.name = 'Aspro-created constraints'
            skyTransparencyMagLimits = {"AT": 3, "UT": 5}
            if acqTSF.ISS_IAS_HMAG < skyTransparencyMagLimits[tel]:
                obConstraints.skyTransparency = 'Variable, thin cirrus'
            else:
                obConstraints.skyTransparency = 'Clear'

            if acqTSF.ISS_IAS_HMAG > 7.5:
                acqTSF.INS_DISP_NAME = "FREE"

            # FIXME: error (OB): "Phase 2 constraints must closely follow what was requested in the Phase 1 proposal.
            # The seeing value allowed for this OB is >= java0x0 arcsec."
            # FIXME REPLACE SEEING THAT IS NO MORE SUPPORTED
            # obConstraints.seeing = 1.0
            # FIXME: default values NOT IN ASPRO!
            # constaints.airmass = 5.0
            # constaints.fli = 1

            # compute dit, ndit, nexp

            # and store computed values in obsTSF
            obsTSF.SEQ_NEXPO = 5
            # obsTSF.NSCANS = 100
            # kappaTSF.SEQ_DOIT=False
            # darkTSF.SEQ_DOIT=True

            # then call the ob-creation using the API if p2container exists.
            if p2container == None:
                ui.addToLog(obTarget.name +
                            " ready for p2 upload (details logged)")
                ui.addToLog(obTarget, False)
                ui.addToLog(obConstraints, False)
                ui.addToLog(acqTSF, False)
                ui.addToLog(obsTSF, False)
                ui.addToLog(kappaTSF, False)
                ui.addToLog(darkTSF, False)
            else:
                self.createPionierOB(p2container, obTarget, obConstraints, acqTSF,
                                     obsTSF, kappaTSF, darkTSF, OBJTYPE, instrumentMode, TEL_COU_GSSOURCE, GSRA, GSDEC,
                                     TEL_COU_MAG, LSTINTERVAL)
                ui.addToLog(obTarget.name + " submitted on p2")

    def formatRangeTable(self):
        rangeTable = self.getRangeTable()
        buffer = ""
        for l in rangeTable.keys():
            buffer += l + "\n"
            for k in rangeTable[l].keys():
                constraint = rangeTable[l][k]
                keys = constraint.keys()
                buffer += ' %30s :' % (k)
                if 'min' in keys and 'max' in keys:
                    buffer += ' %f ... %f ' % (
                        constraint['min'], constraint['max'])
                elif 'list' in keys:
                    buffer += str(constraint['list'])
                elif "spaceseparatedlist" in keys:
                    buffer += ' ' + " ".join(constraint['spaceseparatedlist'])
                if 'default' in keys:
                    buffer += ' (' + str(constraint['default']) + ')'
                else:
                    buffer += ' -no default-'
                buffer += "\n"
        return buffer

    def formatDitTable(self):
        buffer = ' No dit table in use \n'

#        ditTable = self.getDitTable()
#        buffer = '   Tel | Spec |  Pol  |     H   | DIT(s)\n'
#        buffer += '--------------------------------------------------------\n'
#        for tel in ['AT']:
#           for spec in ['GRISM', 'FREE']:
#                for pol in ['IN', 'OUT']:
#                    for i in range(len(ditTable[tel][spec][pol]['DIT'])):
#                        buffer += ' %3s | %4s | %3s | %2s |' % (tel,
#                                                                spec, pol, tel)
#                        buffer += ' %4.1f <K<= %3.1f | %4.1f' % (ditTable[tel][spec][pol]['MAG'][i],
#                                                                 ditTable[tel][spec][
#                                                                     pol]['MAG'][i + 1],
#                                                                 ditTable[tel][spec][pol]['DIT'][i])
#                        buffer += "\n"
#            Hut = ditTable[tel]['Hut']
        return buffer

    def getPionierTemplateName(self, templateType, OBJTYPE):
        objType = "calibrator"
        if OBJTYPE and "SCI" in OBJTYPE:
            objType = "science"
        if OBJTYPE:
            return "_".join((self.getName(), templateType, objType))
        return "_".join((self.getName(), templateType))

    def getPionierObsTemplateName(self, OBJTYPE):
        return self.getPionierTemplateName("obs", OBJTYPE)

    def createPionierOB(
            self, p2container, obTarget, obConstraints, acqTSF, obsTSF, kappaTSF, darkTSF, OBJTYPE, instrumentMode,
            TEL_COU_GSSOURCE, GSRA, GSDEC, TEL_COU_MAG, LSTINTERVAL):

        api = self.facility.getAPI()
        ui = self.ui
        ui.setProgress(0.1)

        # TODO compute value
        VISIBILITY = 1.0

        # everything seems OK
        # create new OB in container:
        # TODO use a common function for next lines
        goodName = re.sub('[^A-Za-z0-9]+', '_', obTarget.name)
        OBS_DESCR = OBJTYPE[0:3] + '_' + goodName + '_PIONIER_' + \
            acqTSF.ISS_BASELINE[0] + '_' + instrumentMode

        ob, obVersion = api.createOB(p2container.containerId, OBS_DESCR)
        ui.addToLog("Getting new ob from p2: ")
        obId = ob['obId']

        # we use obId to populate OB
        ob['obsDescription']['name'] = OBS_DESCR[0:min(len(OBS_DESCR), 31)]
        ob['obsDescription']['userComments'] = self.getA2p2Comments()
        # ob['obsDescription']['InstrumentComments'] = 'AO-B1-C2-E3' #should be
        # a list of alternative quadruplets!

        # copy target info
        targetInfo = obTarget.getDict()
        for key in targetInfo:
            ob['target'][key] = targetInfo[key]

        # copy constraints info
        constraints = obConstraints.getDict()
        for k in constraints:
            ob['constraints'][k] = constraints[k]

        ui.addToLog("Save ob to p2:\n%s" % ob, False)
        ob, obVersion = api.saveOB(ob, obVersion)

        # time constraints if present
        self.saveSiderealTimeConstraints(api, obId, LSTINTERVAL)

        ui.setProgress(0.2)

        # then, attach acquisition template(s)
        tpl, tplVersion = api.createTemplate(obId, 'PIONIER_acq')
        # and put values
        # start with acqTSF ones and complete manually missing ones
        values = acqTSF.getDict()
        # TODO move next updates earlier so it can be tested during checkOb phase
        values.update({'TEL.COU.GSSOURCE': TEL_COU_GSSOURCE,
                       'TEL.COU.ALPHA': GSRA,
                       'TEL.COU.DELTA': GSDEC,
                       'TEL.COU.MAG': round(TEL_COU_MAG, 3)
                       })

        tpl, tplVersion = api.setTemplateParams(obId, tpl, values, tplVersion)
        ui.setProgress(0.3)

        # Put Obs template
        tpl, tplVersion = api.createTemplate(
            obId, self.getPionierObsTemplateName(OBJTYPE))
        ui.setProgress(0.4)
        values = obsTSF.getDict()
        tpl, tplVersion = api.setTemplateParams(obId, tpl, values, tplVersion)
        ui.setProgress(0.5)

        # put Kappa Matrix Template
        tpl, tplVersion = api.createTemplate(obId, 'PIONIER_gen_cal_kappa')
        ui.setProgress(0.6)
        values = kappaTSF.getDict()
        tpl, tplVersion = api.setTemplateParams(obId, tpl, values, tplVersion)
        ui.setProgress(0.7)

        # put Dark Template
        tpl, tplVersion = api.createTemplate(obId, 'PIONIER_gen_cal_dark')
        ui.setProgress(0.8)
        values = darkTSF.getDict()
        tpl, tplVersion = api.setTemplateParams(obId, tpl, values, tplVersion)
        ui.setProgress(0.9)

        # verify OB online
        response, _ = api.verifyOB(obId, True)
        ui.setProgress(1.0)
        self.showP2Response(response, ob, obId)



#!/usr/bin/env python

__all__ = []

from a2p2.chara.gui import CharaUI
from a2p2.facility import Facility
import requests
import logging
import traceback

HELPTEXT = "TODO update this HELP message in a2p2/chara/facility.py"

logger = logging.getLogger(__name__)

class CharaFacility(Facility):

    def __init__(self, a2p2client):
        Facility.__init__(self, a2p2client, "CHARA", HELPTEXT)
        self.charaUI = CharaUI(self)
        self.connected2OB2 = False

    def processOB(self, ob):
        self.a2p2client.ui.addToLog(
            "OB received for '" + self.facilityName + "' interferometer")
        # show ob dict for debug
        self.a2p2client.ui.addToLog(str(ob), False)

        # performs operation
        self.consumeOB(ob)

        # give focus on last updated UI
        self.a2p2client.ui.showFacilityUI(self.charaUI)

    def consumeOB(self, ob):
        # forward message if a server is present in the preferences
        charaServer=self.a2p2client.preferences.getCharaQueueServer()
        if charaServer :
            try:
                r = requests.post(charaServer, json=ob.as_dict())
                msg = ""
                if not self.connected2OB2:
                    try:
                        c=requests.get(charaServer)
                        msg += f"Connection succeeded on OB2 server : {c.json()}\n"
                    except:
                        msg += f"Connection succeded on a non identified OB2 server\n"
                msg+=f"OB sent to remote server queue : {r}"
                self.connected2OB2 = True
            except:
                print(traceback.format_exc())
                msg=f"Can't send OB to remote server queue, please check access to server {charaServer}"
                self.connected2OB2 = False

            self.a2p2client.ui.addToLog(msg)
            self.charaUI.display(msg)

        # display OB
        self.charaUI.displayOB(ob)





__version__ = "0.6.7"

__release_notes__ = {
    # "0.1.6": {
    #    "STATUS":[
    #             "This version get lot of changes and may contain bugs or missing features, please provide any feedback to improve and prepare a better future release !"
    #         ],
    #    "A2P2": [
    #
    #        ],
    #    "VLTI": [
    #
    #        ],
    #    "CHARA": [
    #
    #        ],
    #    "TODO-SCIENCE": [
    #
    #        ],
    #    "TODO-DEV": [
    #
    #        ],
    #    },
     "0.6.7": {
        "STATUS": [
        ],
        "A2P2": [
        ],
        "VLTI": [
        ],
        "CHARA": [
            "Enhance information  while using OB2 server prototype",
        ],
        "TODO-SCIENCE": [
            "Merge AO or GS in a same code section for every instruments",
            "Check DIT table from the last template user manuals (especially MATISSE) and remove PIONIER's one",
            "flag ~important~ keywords which MUST be set in a2p2 code and not leaved to there default values?",
        ],
        "TODO-DEV": [
            "Support multiple period version (two major at least)",
            "Support numlist keyword : eg. SEQ.HWPOFF (done in conf but must be range check compatible)",
            "Unify ob name creation in vlti instrument createOB()",
            "Complete test suite with real p2 submission",
            "Try to read OB in P2 and send them back to Aspro2 as a new obs",
        ],
      }, "0.6.6": {
        "CHARA": [
            "Fix formating of OB supporting array of observability time intervals",
        ],
    },"0.6.5": {
        "CHARA": [
            "Experimental: forward OBs through json payload to the location defined in preference",
        ],
    },"0.6.4": {
        "STATUS": [
        ],
        "A2P2": [
            "Fix import that may break run on windows..."
        ],
    },"0.6.3": {
        "VLTI": [
            "Always use GRAVITY acqTSF.SEQ_INS_SOBJ_HMAG after P110",
        ],
    },"0.6.2": {
        "STATUS": [
        ],
        "A2P2": [
        ],
        "VLTI": [
            "Fix GRAVITY gen acq tsf",
        ],
        "CHARA": [
        ],
    },"0.6.1": {
        "STATUS": [
        ],
        "A2P2": [
        ],
        "VLTI": [
            "Updated configuration for ESO P111",
        ],
        "CHARA": [
        ],
    },"0.6.0": {
        "STATUS": [
        ],
        "A2P2": [
            "Read Extra_informations from last Aspro2 OBXML"
        ],
    },"0.5.3": {
        "STATUS": [
        ],
        "A2P2": [
        ],
        "VLTI": [
            "Update DIT tables to last instrument template (version is stored in the table config to report warning)",
            "Do not stop OB creation if K mag is out of range during DIT calculation but show a warning and ask to proceed or abort",
        ],
        "CHARA": [
        ],
    },"0.4.6": {
        "STATUS": [
        ],
        "A2P2": [
            "Huge speedup of ESO P2 runs treeview",
            "Explicit SAMP unregister reduce risk of SAMP ghosts",
        ],
        "VLTI": [
            "Check consistency between Aspro2 inteferometer period and RUN's IP version",
            "Handle use SEQ.INS.SOBJ.HMAG after P110 instead of SEQ.FI.HMAG",
        ],
        "CHARA": [
        ],
    },"0.4.5": {
        "A2P2": [
            "clarify some text in the default generated preference file",
        ],
        "VLTI": [
            "Change log message : Run filled -> OB transmitted",
            "Missing flux error message enhanced : show associated target name",
            "Handle ALPHA DELTA coordinates of associated guide star",
            "MATISSE: handle Aspro2's frindge tracker mode : None or GRA4MAT",
            "MATISSE: define DPR.CATG  (always was CALIB)",
            "MATISSE: use V band for COU.GS.MAG or try G one on ATs.",
            "GRAVITY: include dual keywords in gravity_rangeTable",
        ],
    },"0.4.4": {
        "A2P2": [
            "add Catalog.piname() to get a pi name for a given jmmc account looking at a given catalog (jmmc.login preference is used without parameter)",
            "Add new preference to put jmmc account credentials",
        ],
    },"0.4.3": {
        "A2P2": [
            "Enhance a2p2.jmmc.models._model so it automagically computes component names >{(())>",
        ],
    },"0.4.2": {
        "A2P2": [
            "Add option to select public CatalogAPI server",
        ],
    },"0.4.1": {
        "A2P2": [
            "Add version alpha of a2p2.fr.webservices.Calliper client",
        ],
    }, "0.4.0": {
        "A2P2": [
            "First basic support of SAMP messages from Aspro for models",
            "Support model compositions in models module",
        ],
    }, "0.3.10": {

        "A2P2": [
            "Fix bug that occurs when user has no preference file",
            "Add new serialisation of a2p2.jmmc.Models"
        ],
    }, "0.3.9": {
        "VLTI": [
            "Bugfix for single CAL SCI",
        ],
    }, "0.3.8": {
        "VLTI": [
            "Enhance CAL SCI sequence : [CAL1] SCI [CAL2 [SCI CAL3 [...] ] ] ",
            "Fix COU_AG_PMA and COU_AG_PMD for MATISSE acq template",
            "Do not throw a dialog for every submitted OBs",
            "Enhance some messages",
            # "BugFix to create a folder on non tutorial accounts"
        ],

    }, "0.3.7": {
        "VLTI": [
            "Revert SEQ.RELOFF.X/Y = 0.0 (same as default) for GRAVITY dual_obs_exp template"
        ],
    }, "0.3.6": {
        "VLTI": [
            "Disable SEQ.RELOFF.X for GRAVITY dual to make OB compliant"
        ],
    }, "0.3.5": {
        "VLTI": [
            "Accept to add calibrator inside a Concatenation container",
            "Use p2.iss.vltitype preference keys to set supported value of instrument's acquisition templates. ( run 'a2p2 -c' )"
        ],
    },
    "0.3.4": {
        "VLTI": [
            "Sync templates with P109",
            "Create OB in selected folder: do not create anymore a folder but create a concatenation for SM if a Run's root is selected."
        ],
    },
    "0.3.3": {
        "A2P2": [
            "Add basic support of Aspro2's model for SAMP interoperability",
        ],
        "VLTI": [
            "Fix missing import for p2api module"
        ],
    },
    "0.3.2": {
        "A2P2": [
            "Improve setup.py that now requires python 3+ and a fresh version of astropy",
        ],
        "VLTI": [
            #            "revert back to p2 requests use which now reuses connections",
        ],
    },
    "0.3.1": {
        "A2P2": [
            "Bug fix for authenticated Catalog access",
        ],
    },
    "0.3.0": {
        "A2P2": [
            "Give a try to embedd some code to interact with JMMC services",
        ],
        "VLTI": [
            #            "revert back to p2 requests use which now reuses connections",
        ],
    },
    "0.2.15": {
        "VLTI": [
            #            "add a wrapper on p2 to make run's tree faster (~2.5x)",
        ],
    },
    "0.2.14": {
        "STATUS": [
            "BugFix: ask for container Name only if one is selected"
        ],
    },
    "0.2.13": {
        "A2P2": [
            "A2P2 is no longer python2 compatible. Hope it will be ok for everybody ? Send an issue else ;)",
            "Fix generated release note order according to semver values",
            "Dry tests done looping on a few OBXML files"
            "Added -c option to a2p2 so we generated a config file ( helps to automatically fill P2 login info & autologin : )"
        ],
        "VLTI": [
            "Conf updated with IPs 106.25",
            "BugFix: OB no more sent to P2 if OB's instrument is not the same than p2 selected container"
        ],
    },
    "0.2.12": {
        "A2P2": [
            "Fix import in main console script"
        ],
    },
    "0.2.11": {
        "A2P2": [
            "enhance setup.py so it install Windows special-cases .exe files"
        ],
    },
    "0.2.10": {
        "A2P2": [
            "Patch bad SAMP url handling on Windows"
        ],
    },
    "0.2.9": {
        "VLTI": [
            "Fix bug that prevent to create any folder or concatenation at RUNS's root"
        ],
    },
    "0.2.8": {
        "A2P2": [
            "Fix release notes order in the GUI",
            "Handle special jmmc account, kindly set by ESO colleagues to perfom future tests as closed as possible to the real UX"
        ],
        "VLTI": [
            "Display instrument package version in the container table",
            "Limit keyword set on P2 only to the modified ones. No more default values from our static config are sent so it enhances compatibility accross multiple Period versions",
        ],
        "CHARA": [
        ],
    }, "0.2.7": {
        "STATUS": [
            "This version get lot of changes and may contain bugs or missing features, please provide any feedback to improve and prepare a better future release !"
        ],
        "A2P2": [
            "Refactor code accross vlti instruments",
            "Fix container selection in P2 tree"
            "Add release notes in the GUI"
        ],
        "VLTI": [
            "Conf updated with IPs 105.18",
            "Add MATISSE support",
            "Change GRAVITY DIT computation",
            "OB constraints autochecked using an instrumentConstraints TSF",
            "Support Concatenations (also shown in the tree panel)",
            "Show type in the container chooser instead of containerID"
        ],
    }, "0.2.6": {
        "VLTI": [
            "Support baseline back again (single one at present)"
        ],
    },
    "0.2.5": {
        "VLTI": [
            "Add missing template name in log",
            "Fix error removing baseline after constraint changes on P2 side. Next a2p2 version should add them back in acq templates"
        ],
    },
    "0.2.4": {
        "VLTI": [
            "Fix bug / wrong keys"
        ],
    },
    "0.2.3": {
        "VLTI": [
            "Hide password in login frame"
        ],
    },
    "0.2.2": {
        "VLTI": [
            "ignore default time constraints computed by Aspro"
        ],
    },
    "0.2.1": {
        "VLTI": [
            "fix support for a list of multiples time constraints"
        ],
    },
    "0.2.0": {
        "VLTI": [
            "bug fix"
        ],
    },
    "0.1.6": {
        "A2P2": [
            "Major code reformating - pep8 compliant"
        ],
        "VLTI": [
            "general config updates",
            "add PIONIER"
        ],
    },
    "0.1.5": {
        "VLTI": [
            "bugfix for dualfield cases"
        ],
    },
    "0.1.4": {
        "A2P2": [
            "fix order of returned fluxes in OB.getFluxes()"
        ]
    },
    "0.1.3": {
        "VLTI": [
            "fix telescope mode computation"
        ]
    },
    "0.1.2": {
        "A2P2": [
            "bugfix for SPLIT polarisation mode detection on GRAVITY",
            "bugfix that displays warning message during DIT calculation in GRAVITY LR mode",
            "enhancement of out of bound exception message of DIT calculation method"
        ]
    },
    "0.1.1": {
        "A2P2": [
            "muti-faciliy",
            "multi-VltiInstruments",
            "json VltiConfig (templates+dit tables)"
        ]
    }
}



#!/usr/bin/env python

__all__ = []

import logging
import json

from .utils import JmmcAPI
from . import PRODLABEL

logger = logging.getLogger(__name__)

class CallIper():
    """ Call IPER using its 'API' through simple methods.

    from a2p2.jmmc import CallIper
    calliper = CallIper()
    diskresult   = caliper.runDiskFit(pathname)
    ud = diskresult["diameter"]

    """
    def __init__(self, prod=False, apiUrl=None):
        self.prod = prod
        self.serviceName = "callIper"

        # Manage prod & preprod or user provided access points
        if apiUrl:
            self.apiUrl = apiUrl  # trust given url as callIperAPI if value is provided
        elif self.prod:
            self.apiUrl = ""  # no api in production yet
            raise Warning("sorry, no api for production yet")
        else:
            self.apiUrl = "http://apps.jmmc.fr/~mellag/LITproWebService/run.php"

        self.api = JmmcAPI(self.apiUrl)

        logger.info("Create calliper client to access '%s' (%s API at %s)" %
                    (self.serviceName, PRODLABEL[self.prod], self.api.rootURL))

    # API will evolve in the future with parameter to select fitter, model and data to fit...
    def runDiskFit(self, oifitsFilename, jsonOutput=True):
        """ Ask CallIper to fit given data for a simple disk """
        files = {'userfile': (oifitsFilename, open(oifitsFilename, 'rb'))}
        data = {'method': 'runJsonFit'}
        res=self.api._post("", data=data, files=files)
        if jsonOutput:
            return json.loads(res)
        return res



#!/usr/bin/env python

__all__ = ['facility', 'instrument', 'gui', 'samp', 'client']

from .version import __release_notes__
from .version import __version__


from . import client
from . import facility
from . import gui
from . import instrument
from . import samp
from .client import A2p2Client



#!/usr/bin/env python

__all__ = []

import json
import logging
import xml.etree.ElementTree as ET
import json

from .utils import JmmcAPI
from a2p2.ob import etree_to_dict

logger = logging.getLogger(__name__)


#
# first release done for SAMP interoperability with Aspro2
#

def modelsFromXml(xmlmodel):
    """ Returns a python object representing JSON LIKE model(s) from given XML."""
    # read xml models from Aspro2 and convert them to a JSON Model
    e = ET.fromstring(xmlmodel)
    _remove_namespace(e, "http://www.jmmc.fr/jmcs/models/0.1")
    container = etree_to_dict(e)
    modellist=container['model']['model']
    if not isinstance(modellist, list):
        modellist=[modellist]

    models=[]
    for m in modellist:
        model={}
        for p in m['parameter']:
            model[p['type']]=p['value']
        model['name']=m['name']
        model['type']=m['type']
        models.append(model)

    return json.dumps(models)


def _remove_namespace(doc, namespace):
    """Remove namespace in the passed document in place.
    from https://homework.nwsnet.de/releases/45be/"""
    ns = u'{%s}' % namespace
    nsl = len(ns)
    for elem in doc.getiterator():
        if elem.tag.startswith(ns):
            elem.tag = elem.tag[nsl:]


def _model(models, output_mode=None):
    """ Returns a serialisation of given model(s) to XML by default or another format given to output_mode value (only jsonlike at present time)."""

    # deserialize if models is a string
    if isinstance(models, str):
        models=json.loads(models)

    if output_mode:
        return models
    else :
        return _xml_model(models)

def _xml_model(models, position=1):
    """ Rough conversion of dict(s) to xml (Aspro2 namespaces)
    if name key is not provided, position value will be used to compute suffix(es)"""

    if isinstance(models, list):
        modellist=[]
        pos=1
        for m in models:
            modellist.append(_xml_model(m, pos))
            pos+=1 # do not use models.index(m) since same str components would get the same index
        return "".join(modellist)

    model = models # models is now a single element
    modeltype=model["type"]
    if "name" in model.keys():
        modelname=model["name"]
    else:
        modelname=modeltype+str(position)


    params=""
    paramNames=[ k for k in model.keys() if not k in ("type", "name")]
    # at present time we must use Aspro2's approach with the same xml namespaces
    # see
    for p in paramNames:
        params+=f"""    <tm:parameter name="{modelname}_{p}" type="{p}"><value>{model[p]}</value></tm:parameter>\n"""

    return f"""<tm:model name="{modelname}" type="{modeltype}">\n{params}</tm:model>\n"""


class Models():
    """ Get analytical model's representations (in sync with Aspro2's ones).
    """
    SAMP_UCD_MODEL="meta.code.class;meta.modelled" # use it for colums that would be filled by models below

# generated code below from an Asprox file that contains a single star with list of all supported models
# cd a2p2/jmmc
# xml sel -o "from .models import _model" -n -n -b -t -m "//tm:model" -o "def " -v "@type" -o "( name, " -m "tm:parameter" -v "@type" -o "=" -v "value" -o ", " -b -o " output_mode=None):" -n -o '    """ ' -v "desc" -o ' """ ' -n -n -o '    return _model({' -m "tm:parameter" -o '"' -v "@type" -o '" : ' -v "@type" -o ", " -b -o '"name":name, "type":"' -v "@type" -o '"}, output_mode)' -n -n models_template.asprox > generated_models.py
# xml sel -t -o "    from .generated_models import " -m "//tm:model" -v "@type" -o ", "  -b -n models_template.asprox
# cd -
    from .generated_models import punct, disk, elong_disk, flatten_disk, circle, ring, elong_ring, flatten_ring, gaussian, elong_gaussian, flatten_gaussian, limb_quadratic



#!/usr/bin/env python

__all__ = []

from astropy.samp import SAMPIntegratedClient
from os import sep
import logging

logger = logging.getLogger(__name__)

class Receiver(object):

    def __init__(self, client):
        self.client = client
        self.received = False

    def receive_call(self, private_key, sender_id, msg_id, mtype, params, extra):
        self.mtype=mtype
        self.params = params
        self.received = True
        self.client.reply(
            msg_id, {"samp.status": "samp.ok", "samp.result": {}})

    def receive_notification(self, private_key, sender_id, mtype, params, extra):
        self.mtype=mtype
        self.params = params
        self.received = True

    def clear(self):
        self.received = False
        self.params = None

    def get_last_message(self):
        pass  # TODO handle here a buffer ...


class A2p2SampClient():
    # TODO watch hub disconnection

    def __init__(self):
        self.sampClient = SAMPIntegratedClient(
            "A2P2 samp relay")  # TODO get title from main program class instead of HardCoded value

    def __del__(self):
        self.disconnect()

    def connect(self):
        self.sampClient.connect()
        # an error is thrown here if no hub is present

        # TODO get samp client name and display it in the UI
        # TODO !! declare mtypes as enum

        # Instantiate the receiver
        self.r = Receiver(self.sampClient)
        # Listen for any instructions to load a table
        self.sampClient.bind_receive_call("ob.load.data", self.r.receive_call)
        self.sampClient.bind_receive_notification(
            "ob.load.data", self.r.receive_notification)
        # Listen for Model related instructions
        self.sampClient.bind_receive_call("fr.jmmc.litpro.start.setting",self.r.receive_call)
        self.sampClient.bind_receive_notification(
            "fr.jmmc.litpro.start.setting",self.r.receive_notification)

    def disconnect(self):
        self.sampClient.disconnect()

    def is_connected(self):
        # Workarround the 'non' reliable is_connected attribute
        # this helps to reconnect after hub connection lost
        try:
            return self.sampClient.is_connected and (self.sampClient.ping() or self.sampClient.is_connected)
        except:
            # consider connection refused exception as not connected state
            return False

    def get_status(self):
        if self.is_connected():
            return "connected [%s]" % (self.sampClient.get_public_id())
        else:
            return "not connected"

    def get_public_id(self):
        return self.sampClient.get_public_id()

    def has_message(self):
        return self.is_connected() and self.r.received

    def clear_message(self):
        return self.r.clear()

    def has_ob_message(self):
        return "ob.load.data" in self.r.mtype

    def get_ob_url(self):
        url = self.r.params['url']
        if url.startswith("file:///"):
            if sep == '/':
                return url[7:]
            else:
                return url[8:] # Do not leave leading / on windows machines for file:///C:/Users....
        elif url.startswith("file:/"):  # work arround bugged file urls on *nix
            return url[5:]
        return url

    def has_model_message(self):
        return "fr.jmmc.litpro.start.setting" in self.r.mtype

    def get_model(self):
        return self.r.params['model']





#!/usr/bin/env python

__all__ = []

import re

import numpy as np

from a2p2.vlti.instrument import OBConstraints
from a2p2.vlti.instrument import OBTarget
from a2p2.vlti.instrument import TSF
from a2p2.vlti.instrument import VltiInstrument

HELPTEXT = """
Please define Gravity instrument help in a2p2/vlti/gravity.py
"""


class Gravity(VltiInstrument):

    def __init__(self, facility):
        VltiInstrument.__init__(self, facility, "GRAVITY")

    def checkOB(self, ob, p2container=None):

        ui = self.ui

        instrumentConfiguration = ob.instrumentConfiguration
        BASELINE = ob.interferometerConfiguration.stations
        # Compute tel = UT or AT
        # TODO move code into common part
        if "U" in BASELINE:
            tel = "UT"
        else:
            tel = "AT"

        instrumentMode = instrumentConfiguration.instrumentMode

        # Retrieve SPEC_RES and POL info from instrumentMode

        # We could force ins_spec_res to be default but prefer a bad keyword value
        # that throws a Value Error if instrumentMode does not start with an accepted one
        # ins_spec_res = self.getDefault("GRAVITY_gen_acq.tsf", "INS.SPEC.RES")
        ins_spec_res = "BAD_SPEC_RES"
        for res in self.getRange("GRAVITY_gen_acq.tsf", "INS.SPEC.RES"):
            if res in instrumentMode[0:len(res)]:
                ins_spec_res = res

        if "COMBINED" in instrumentMode:
            ins_pol = "OUT"
        else:
            ins_pol = 'IN'

        for observationConfiguration in self.getSequence(ob):

            # create keywords storage objects
            acqTSF = TSF(self, "GRAVITY_gen_acq.tsf")
            obsTSF = TSF(self, "GRAVITY_single_obs_exp.tsf")
            # alias for
            # ,GRAVITY_single_obs_calibrator.tsf,GRAVITY_dual_obs_exp.tsf,GRAVITY_dual_obs_calibrator.tsf")
            obTarget = OBTarget()
            obConstraints = OBConstraints(self)

            # set common properties if any
            acqTSF.INS_SPEC_RES = ins_spec_res
            acqTSF.INS_FT_POL = ins_pol
            acqTSF.INS_SPEC_POL = ins_pol

            if 'SCIENCE' in observationConfiguration.type:
                OBJTYPE = 'SCIENCE'
            else:
                OBJTYPE = 'CALIBRATOR'

            scienceTarget = observationConfiguration.SCTarget

            # define target
            acqTSF.SEQ_INS_SOBJ_NAME = scienceTarget.name.strip()
            obTarget.name = acqTSF.SEQ_INS_SOBJ_NAME.replace(
                ' ', '_')  # allowed characters: letters, digits, + - _ . and no spaces
            obTarget.ra, obTarget.dec = self.getCoords(scienceTarget)
            obTarget.properMotionRa, obTarget.properMotionDec = self.getPMCoords(
                scienceTarget)

            # Set baseline  interferometric array code (should be a keywordlist)
            acqTSF.ISS_BASELINE = [self.getBaselineCode(BASELINE)]

            self.checkIssVltiType(acqTSF)

            # define some default values
            acqTSF.SEQ_INS_SOBJ_DIAMETER = float(self.get(scienceTarget, "DIAMETER", 0.0))

            # TODO compute value
            VISIBILITY = 1.0
            acqTSF.SEQ_INS_SOBJ_VIS = VISIBILITY

            # Retrieve Fluxes
            acqTSF.COU_GS_MAG = self.getFlux(scienceTarget, "V")
            acqTSF.SEQ_INS_SOBJ_MAG = self.getFlux(scienceTarget, "K")
            if ob.getPeriod() >= 110: # P111 use back again FI_HMAG
                acqTSF.SEQ_INS_SOBJ_HMAG = self.getFlux(scienceTarget, "H")
            else:
                acqTSF.SEQ_FI_HMAG = self.getFlux(scienceTarget, "H")

            # setup some default values, to be changed below
            GSRA = '00:00:00.000'
            GSDEC = '00:00:00.000'
            dualField = False
            dualFieldDistance = 0.0  # needed as must exist for the argument list of createGravityOB


            # if FT Target is not ScTarget, we are in dual-field (TBD)
            ftTarget = ob.get(observationConfiguration, "FTTarget")
            if ftTarget != None:
                acqTSF.SEQ_FT_ROBJ_NAME = ftTarget.name
                FTRA, FTDEC = self.getCoords(ftTarget)
                # no PMRA, PMDE for FT !!
                if ob.getPeriod() == 110: # P111 use back again FI_HMAG
                    acqTSF.SEQ_INS_SOBJ_HMAG = self.getFlux(ftTarget, "H")
                else:
                    acqTSF.SEQ_FI_HMAG = self.getFlux(ftTarget, "H")
                SEQ_FI_HMAG = float(ftTarget.FLUX_H)
                # just to say we must treat the case there
                # is no FT Target
                acqTSF.SEQ_FT_ROBJ_MAG = self.getFlux(ftTarget, "K")
                #acqTSF.SEQ_FT_ROBJ_DIAMETER = 0.0  # FIXME
                SEQ_FT_ROBJ_VIS = 1.0  # FIXME
                dualField = True

                # test distance in dual field mode
                if tel == "UT":
                    SCtoREFmaxDist = 2000
                    SCtoREFminDist = 0  # before P103: 400
                else:
                    SCtoREFminDist = 0  # before P103: 1500
                    SCtoREFmaxDist = 4000
                # compute x,y between science and ref beams:
                dualFieldDistance = self.getSkyDiff(
                    obTarget.ra, obTarget.dec, FTRA, FTDEC)
                if np.abs(dualFieldDistance[0]) < SCtoREFminDist:
                    raise ValueError("Dual-Field distance of two stars is  < " + str(
                        SCtoREFminDist) + " mas, Please Correct.")
                elif np.abs(dualFieldDistance[0]) > SCtoREFmaxDist:
                    raise ValueError("Dual-Field distance of two stars is  > " + str(
                        SCtoREFmaxDist) + " mas, Please Correct.")
                acqTSF.SEQ_INS_SOBJ_X=dualFieldDistance[0]
                acqTSF.SEQ_INS_SOBJ_Y= dualFieldDistance[1]

                acqTSF.SEQ_FT_ROBJ_VIS= SEQ_FT_ROBJ_VIS
                acqTSF.SEQ_FT_MODE= "AUTO"


            # AO target
            aoTarget = ob.get(observationConfiguration, "AOTarget")
            if aoTarget != None:
                AONAME = aoTarget.name
                acqTSF.COU_AG_GSSOURCE = 'SETUPFILE'  # since we have an AO
                acqTSF.COU_AG_ALPHA, acqTSF.COU_AG_DELTA = self.getCoords(aoTarget, requirePrecision=False)
                acqTSF.COU_AG_PMA, acqTSF.COU_AG_PMD = self.getPMCoords(aoTarget)
                # Case of CIAO to be implemented...based on v and k magnitudes?
                acqTSF.COU_GS_MAG = float(aoTarget.FLUX_V)

            # Guide Star
            gsTarget = ob.get(observationConfiguration, 'GSTarget')
            if gsTarget != None and aoTarget == None:
                acqTSF.COU_AG_SOURCE = 'SETUPFILE'  # since we have an GS
                acqTSF.COU_AG_ALPHA, acqTSF.COU_AG_DELTA = self.getCoords(gsTarget, requirePrecision=False)
                acqTSF.COU_AG_PMA, acqTSF.COU_AG_PMD = self.getPMCoords(gsTarget)
                acqTSF.COU_GS_MAG = float(gsTarget.FLUX_V)

            # LST interval
            try:
                obsConstraint = observationConfiguration.observationConstraints
                LSTINTERVAL = obsConstraint.LSTinterval
            except:
                LSTINTERVAL = None

            # Constraints
            obConstraints.name = 'Aspro-created constraints'
            skyTransparencyMagLimits = {"AT": 3, "UT": 5}
            if acqTSF.SEQ_INS_SOBJ_MAG < skyTransparencyMagLimits[tel]:
                obConstraints.skyTransparency = 'Variable, thin cirrus'
            else:
                obConstraints.skyTransparency = 'Clear'
            # FIXME: error (OB): "Phase 2 constraints must closely follow what was requested in the Phase 1 proposal.

            # The seeing value allowed for this OB is >= java0x0 arcsec."
            # FIXME REPLACE SEEING THAT IS NO MORE SUPPORTED
            # obConstraints.seeing = 1.0

            # FIXME: default values NOT IN ASPRO!
            # constaints.airmass = 5.0
            # constaints.fli = 1

            # compute dit, ndit, nexp
            dit = self.getDit(tel, acqTSF.INS_SPEC_RES, acqTSF.INS_SPEC_POL,
                              acqTSF.SEQ_INS_SOBJ_MAG, dualField, targetName=scienceTarget.name.strip(),  showWarning=(p2container == None))
            ndit = 300 / dit
            if ndit < 10:
                ndit = 10
            if ndit > 300:
                ndit = 300
            ndit = int(ndit)  # must be integer
            exptime = int(ndit * dit + 40)  # 40 sec overhead by exp
            nexp = (1800 - 900) / exptime
            nexp = int(nexp)
            ui.addToLog(
                'number of exposures to reach 1800 s per OB is ' + str(nexp))
            if nexp < 3:
                nexp = 3  # min is O S O
                # recompute ndit
                exptime = (1800 - 900) / nexp
                ndit = (exptime - 40) / dit
                ndit = int(ndit)
                if ndit < 10:
                    ndit = 10
                    ui.addToLog(
                        "**Warning**, OB NDIT has been set to min value=%d, but OB will take longer than 1800 s" % (
                            ndit))
            nexp %= 40
            sequence = 'O S O O S O O S O O S O O S O O S O O S O O S O O S O O S O O S O O S O O S O O'
            my_sequence = sequence[0:2 * nexp]
            # and store computed values in obsTSF
            obsTSF.DET2_DIT = str(dit)
            obsTSF.DET2_NDIT_OBJECT = ndit
            obsTSF.DET2_NDIT_SKY = ndit
            obsTSF.SEQ_OBSSEQ = my_sequence
            obsTSF.SEQ_SKY_X = 2000
            obsTSF.SEQ_SKY_Y = 2000

            # then call the ob-creation using the API if p2container exists.
            if p2container == None:
                ui.addToLog(obTarget.name +
                            " ready for p2 upload (details logged)")
                ui.addToLog(obTarget, False)
                ui.addToLog(obConstraints, False)
                ui.addToLog(acqTSF, False)
                ui.addToLog(obsTSF, False)
            else:
                self.createGravityOB(p2container, obTarget, obConstraints, acqTSF, obsTSF, OBJTYPE, instrumentMode,
                                    dualField, LSTINTERVAL)
                ui.addToLog(obTarget.name + " submitted on p2")

    def formatRangeTable(self):
        rangeTable = self.getRangeTable()
        buffer = ""
        for l in rangeTable.keys():
            buffer += l + "\n"
            for k in rangeTable[l].keys():
                constraint = rangeTable[l][k]
                keys = constraint.keys()
                buffer += ' %30s :' % (k)
                if 'min' in keys and 'max' in keys:
                    buffer += ' %f ... %f ' % (
                        constraint['min'], constraint['max'])
                elif 'list' in keys:
                    buffer += str(constraint['list'])
                elif "spaceseparatedlist" in keys:
                    buffer += ' ' + " ".join(constraint['spaceseparatedlist'])
                if 'default' in keys:
                    buffer += ' (' + str(constraint['default']) + ')'
                else:
                    buffer += ' -no default-'
                buffer += "\n"
        return buffer

    def formatDitTable(self):
        ditTable = self.getDitTable()
        buffer = '    Mode     |Spec |  Pol  |Tel |       K       | DIT(s)\n'
        buffer += '--------------------------------------------------------\n'
        for tel in ['AT']:
            for spec in ['LOW', 'MED', 'HIGH']:
                for pol in ['OUT', 'IN']:
                    for i in range(len(ditTable[tel][spec][pol]['DIT'])):
                        buffer += 'Single Field | %4s | %3s | %2s |' % (
                            spec, pol, tel)
                        buffer += ' %4.1f <K<= %3.1f | %4.1f' % (ditTable[tel][spec][pol]['MAG'][i],
                                                                 ditTable[tel][spec][
                                                                     pol]['MAG'][i + 1],
                                                                 ditTable[tel][spec][pol]['DIT'][i])
                        buffer += "\n"
            Kdf = ditTable[tel]['Kdf']
            Kut = ditTable[tel]['Kut']
            buffer += ' Dual Field  |  all | all | %2s | Kdf = K - %.1f |  -\n' % (
                tel, Kdf)
            tel = "UT"
            buffer += 'Single Field |  all | all | %2s | Kdf = K - %.1f |  -\n' % (
                tel, Kut)
            buffer += ' Dual Field  |  all | all | %2s | Kdf = K - %.1f |  -\n' % (
                tel, Kut + Kdf)
        return buffer

    def getGravityTemplateName(self, templateType, dualField, OBJTYPE):
        objType = "calibrator"
        if OBJTYPE and "SCI" in OBJTYPE:
            objType = "exp"
        field = "single"

        if dualField:
            field = "dual"
        if OBJTYPE:
            return "_".join((self.getName(), field, templateType, objType))
        return "_".join((self.getName(), field, templateType))

    def getGravityObsTemplateName(self, OBJTYPE, dualField=False):
        return self.getGravityTemplateName("obs", dualField, OBJTYPE)

    def getGravityAcqTemplateName(self, dualField=False, OBJTYPE=None):
        return self.getGravityTemplateName("acq", dualField, OBJTYPE)

    def createGravityOB(
            self, p2container, obTarget, obConstraints, acqTSF, obsTSF, OBJTYPE, instrumentMode,
            dualField, LSTINTERVAL):

        api = self.facility.getAPI()
        ui = self.ui
        ui.setProgress(0.1)

        # everything seems OK
        # create new OB in container:
        # TODO use a common function for next lines
        goodName = re.sub('[^A-Za-z0-9]+', '_', acqTSF.SEQ_INS_SOBJ_NAME)
        OBS_DESCR = OBJTYPE[0:3] + '_' + goodName + '_GRAVITY_' + \
            acqTSF.ISS_BASELINE[0] + '_' + instrumentMode

        ob, obVersion = api.createOB(p2container.containerId, OBS_DESCR)
        obId = ob['obId']

        # we use obId to populate OB
        ob['obsDescription']['name'] = OBS_DESCR[0:min(len(OBS_DESCR), 31)]
        ob['obsDescription']['userComments'] = self.getA2p2Comments()
        # ob['obsDescription']['InstrumentComments'] = 'AO-B1-C2-E3' #should be
        # a list of alternative quadruplets!

        # copy target info
        targetInfo = obTarget.getDict()
        for key in targetInfo:
            ob['target'][key] = targetInfo[key]

        # copy constraints info
        constraints = obConstraints.getDict()
        for k in constraints:
            ob['constraints'][k] = constraints[k]

        ui.addToLog("Save ob to p2:\n%s" % ob, False)
        ob, obVersion = api.saveOB(ob, obVersion)

        # time constraints if present
        self.saveSiderealTimeConstraints(api, obId, LSTINTERVAL)

        ui.setProgress(0.2)

        # then, attach acquisition template(s)
        tpl, tplVersion = api.createTemplate(
            obId, self.getGravityAcqTemplateName(dualField=dualField))
        # and put values
        # start with acqTSF ones and complete manually missing ones
        values = acqTSF.getDict()
        tpl, tplVersion = api.setTemplateParams(obId, tpl, values, tplVersion)
        ui.setProgress(0.3)

        tpl, tplVersion = api.createTemplate(
            obId, self.getGravityObsTemplateName(OBJTYPE, dualField))
        ui.setProgress(0.4)

        # put values. they are the same except for dual obs science (?)
        values = obsTSF.getDict()
        # TODO move next updates earlier so it can be tested during checkOb phase
        if dualField and OBJTYPE == 'SCIENCE':
            values.update({'SEQ.RELOFF.X': [0.0], 'SEQ.RELOFF.Y': [0.0]})

        tpl, tplVersion = api.setTemplateParams(obId, tpl, values, tplVersion)
        ui.setProgress(0.5)

        # verify OB online
        response, _ = api.verifyOB(obId, True)
        ui.setProgress(1.0)
        self.showP2Response(response, ob, obId)



#!/usr/bin/env python
from __future__ import with_statement

import traceback
from argparse import ArgumentParser


def main():
    """Main method to start a2p2 program."""
    parser = ArgumentParser(
        description='Move your Aspro2 observation details to an observatory proposal database')
    # parser.add_argument('-c', '--config', action='store_true', help='show instruments and remote service configurations.')
    parser.add_argument('-f', '--fakeapi', action='store_true',
                        help='fake API to avoid remote connection (dev. only).')
    parser.add_argument('-v', '--verbose', action='store_true', help='Verbose')
    parser.add_argument('-c', '--createprefs', action='store_true', help='Create preferences file')

    args = parser.parse_args()

    from . import A2p2Client

    if args.createprefs:
        A2p2Client.createPreferencesFile()
        exit()

    try:
        with A2p2Client(args.fakeapi, args.verbose) as a2p2c:
            # if  args.config:
            #    print(a2p2c)
            # else:
            #    a2p2c.run()
            a2p2c.run()

    except Exception as e:
        if True or args.verbose:
            traceback.print_exc()
        else:
            print('ERROR: %s' % str(e))


if __name__ == '__main__':
    main()
